/**
 * DB utilities
 */
package edu.cmu.fantango.util;

import java.sql.*;
import javax.sql.*;
import javax.naming.*;

/**
 * Utility class to handle DB related codes.
 */
public class DBUtil
{
    /**
     *	Datasource string.
     */
	private static final String dsName = "java:/MySqlDS";


    /**
     *	Constructor made private in ordert disable instantiation.
     */
    private DBUtil ( ) { }


	/**
	 *	Returns datasource.
	 *
	 *	@throws DBSystemException
	 *	@returns Named datasource
	 */
    public static DataSource getDataSource ( ) throws DBSystemException
    {
        System.out.println ( "::DBUtil:: (getDataSource)" );
        try
        {
            Context ctx = new InitialContext();
            return (DataSource)ctx.lookup(dsName);
        }
        catch ( NamingException ne )
        {
            throw new DBSystemException ( "Naming Exception while getting Data source : \n" + ne );
        }
    }


	/**
	 *	Returns a connection.
	 *
	 *	@throws DBSystemException
	 *	@returns Established datasource connection
	 */
    public static Connection getDBConnection ( ) throws DBSystemException
    {
        System.out.println ( "::DBUtil:: (getDBConnection)" );
        try
        {
            return getDataSource ( ).getConnection ( );
        }
        catch ( SQLException se )
        {
            throw new DBSystemException ( "SQL Exception while getting DB connection : \n" + se );
        }
    }


	/**
	 *	Closes a given connection.
	 *
     *	@param stmt Given DB connection
	 *	@throws DBSystemException
	 */
    public static void closeConnection ( Connection dbConnection ) throws DBSystemException
    {
        System.out.println ( "::DBUtil:: (closeConnection)" );
        try
        {
            if ( dbConnection != null && !dbConnection.isClosed ( ) )
            {
                dbConnection.close ( );
            }
        }
        catch ( SQLException se )
        {
            throw new DBSystemException ( "SQL Exception while closing DB connection : \n" + se );
        }
    }


	/**
	 *	Closes a given statement.
	 *
     *	@param stmt Given statement
	 *	@throws DBSystemException
	 */
    public static void closeStatement ( Statement stmt ) throws DBSystemException
    {
        System.out.println ( "::DBUtil:: (closeStatement)" );
        try
        {
            if ( stmt != null )
            {
                stmt.close ( );
            }
        }
        catch ( SQLException se )
        {
            throw new DBSystemException ( "SQL Exception while closing Statement : \n" + se );
        }
    }


	/**
	 *	Closes a given result set.
	 *
     *	@param result Given result set
	 *	@throws DBSystemException
	 */
    public static void closeResultSet ( ResultSet result ) throws DBSystemException
    {
        System.out.println ( "::DBUtil:: (closeResultSet)" );
        try
        {
            if ( result != null )
            {
                result.close ( );
            }
        }
        catch ( SQLException se )
        {
            throw new DBSystemException ( "SQL Exception while closing Result Set : \n" + se );
        }
    }


	/**
	 *	Returns the utility's field delimiter for query returns.
	 *
	 *	@returns Field delimiter
	 */
	public static String getFieldDelimiter ( )
	{
		return "__";
	}


	/**
	 *	Returns the utility's row delimiter for query row returns.
	 *
	 *	@returns Row delimiter
	 */
	public static String getRowDelimiter ( )
	{
		return "_";
	}


	/**
	 *	Gets a given query result.
	 *
     *	@param query SQL query
	 *	@returns Result set string array of rows
	 */
    public static String[] getQueryResult ( String query )
    {
        System.out.println ( "::DBUtil:: (getQueryResult) Begin for: " + query + "." );
        Connection conn = null;
        Statement stmt = null;
        ResultSet result = null;

        System.out.println ( "::DBUtil:: (getQueryResult) Starting Datasource access." );

        try
        {
            conn = getDBConnection ( );
            stmt = conn.createStatement ( );
            result = stmt.executeQuery ( query );
        	System.out.println ( "::DBUtil:: (getQueryResult) Executed query." );
     		ResultSetMetaData rsmd = result.getMetaData();
     		int cols = rsmd.getColumnCount();

			result.last ( );
			int rows = result.getRow ( );
			result.first ( );
			String []returnString = new String[rows];
			int i, j;
			j = 0;

			do
			{
				returnString[j] = "";
				for ( i = 1; i <= cols; i++ )
					returnString[j] = returnString[j] + result.getString ( i ) + getFieldDelimiter ( );
				++j;
			} while ( result.next ( ) );

        	System.out.println ( "::DBUtil:: (getQueryResult) Returning: " + returnString );
            return returnString;
        }
        catch ( Exception e )
        {
            e.printStackTrace ( );
        }
		finally
		{
			closeConnection ( conn );
			closeStatement ( stmt );
			closeResultSet ( result );
		}

        System.out.println ( "::DBUtil:: (getQueryResult) Error, returning null." );

        return null;
	}
}
